import axios from 'axios';
import {checkEndpoint, urlEndpoint, urlPath} from "./config";


export class CPU {
    /**
     * Retrieves the number of logical CPU cores on the system.
     *
     * Uses the `node_cpu_seconds_total` metric grouped by `cpu` label.
     *
     * @returns {Promise<number | null>} The number of logical CPUs, or null if unavailable.
     */
    static async getLogicalCpuCount(): Promise<number | null> {
        checkEndpoint();
        try {
            const query = 'count(count by (cpu) (node_cpu_seconds_total))';
            const response = await axios.get(`${urlEndpoint}${urlPath}`, { params: { query } });
            const result = response.data.data.result;
            if (!result || result.length === 0) return null;
            return parseInt(result[0].value[1], 10);
        } catch (error) {
            console.error('Failed to fetch logical CPU count:', (error as Error).message);
            return null;
        }
    }

    /**
     * Calculates the average CPU load (usage) as a percentage over a 1-minute rate window.
     *
     * Uses the `node_cpu_seconds_total` metric, subtracting idle time.
     *
     * @returns {Promise<number | null>} CPU load percentage (0–100), or null if unavailable.
     */
    static async getCpuLoad(): Promise<number | null> {
        checkEndpoint()
        try {
            const response = await axios.get(`${urlEndpoint}${urlPath}`, {
                params: {
                    query: `100 - (avg by (instance) (rate(node_cpu_seconds_total{mode="idle"}[1m])) * 100)`
                }
            });

            const result = response.data.data.result;
            if (result.length === 0) return null;
            const load = parseFloat(result[0].value[1]);
            return load;
        } catch (err) {
            console.error('Error querying Prometheus:', (err as Error).message);
            return null;
        }
    }

    /**
     * Retrieves the average CPU temperature in Celsius.
     *
     * Uses the `node_hwmon_temp_celsius` metric.
     *
     * @returns {Promise<number | null>} The average CPU temperature in °C, or null if unavailable.
     */
    static async getAverageTemp(): Promise<number | null> {
        checkEndpoint();
        try {
            const promQuery = 'avg(node_hwmon_temp_celsius)';

            const response = await axios.get(`${urlEndpoint}${urlPath}`, {
                params: { query: promQuery }
            });

            const result = response.data.data.result;

            if (!result || result.length === 0) {
                console.warn('No CPU temperature data found in Prometheus');
                return null;
            }

            const avgTempStr = result[0].value[1];
            const avgTemp = parseFloat(avgTempStr);

            if (isNaN(avgTemp)) return null;

            return parseFloat(avgTemp.toFixed(1));
        } catch (error) {
            console.error('Failed to fetch CPU temperatures from Prometheus:', (error as Error).message);
            return null;
        }
    }

    /**
     * Retrieves the average minimum CPU clock speed in GHz.
     *
     * Uses the `node_cpu_scaling_frequency_min_hertz` metric.
     *
     * @returns {Promise<number | null>} The average minimum CPU clock in GHz, or null if unavailable.
     */
    static async getMinCpuClockGHz(): Promise<number | null> {
        checkEndpoint();
        try {
            const query = 'avg(node_cpu_scaling_frequency_min_hertz)';
            const response = await axios.get(`${urlEndpoint}${urlPath}`, {
                params: { query }
            });

            const result = response.data.data.result;
            if (!result || result.length === 0) {
                console.warn('No min CPU frequency data found in Prometheus');
                return null;
            }

            const freqGHz = parseFloat(result[0].value[1]) / 1_000_000_000;
            return parseFloat(freqGHz.toFixed(2));
        } catch (error) {
            console.error('Failed to fetch min CPU frequency from Prometheus:', (error as Error).message);
            return null;
        }
    }
    /**
     * Retrieves the average maximum CPU clock speed in GHz.
     *
     * Uses the `node_cpu_scaling_frequency_max_hertz` metric.
     *
     * @returns {Promise<number | null>} The average maximum CPU clock in GHz, or null if unavailable.
     */
    static async getMaxCpuClockGHz(): Promise<number | null> {
        checkEndpoint();
        try {
            const query = 'avg(node_cpu_scaling_frequency_max_hertz)';
            const response = await axios.get(`${urlEndpoint}${urlPath}`, {
                params: { query }
            });

            const result = response.data.data.result;
            if (!result || result.length === 0) {
                console.warn('No max CPU frequency data found in Prometheus');
                return null;
            }

            const freqGHz = parseFloat(result[0].value[1]) / 1_000_000_000;
            return parseFloat(freqGHz.toFixed(2));
        } catch (error) {
            console.error('Failed to fetch max CPU frequency from Prometheus:', (error as Error).message);
            return null;
        }
    }



    /**
     * Retrieves the average current CPU clock speed in GHz.
     *
     * Uses the `node_cpu_scaling_frequency_hertz` metric.
     *
     * @returns {Promise<number | null>} The average current CPU clock in GHz, or null if unavailable.
     */
    static async getCurrentCpuClockGHz(): Promise<number | null> {
        checkEndpoint();
        try {
            const query = 'avg(node_cpu_scaling_frequency_hertz)';
            const response = await axios.get(`${urlEndpoint}${urlPath}`, {
                params: { query }
            });

            const result = response.data.data.result;
            if (!result || result.length === 0) {
                console.warn('No current CPU frequency data found in Prometheus');
                return null;
            }

            const freqGHz = parseFloat(result[0].value[1]) / 1_000_000_000;
            return parseFloat(freqGHz.toFixed(2));
        } catch (error) {
            console.error('Failed to fetch current CPU frequency from Prometheus:', (error as Error).message);
            return null;
        }
    }
}
