import axios from 'axios';
import {urlEndpoint, checkEndpoint, urlPath} from './config';

export class RAM {
    /**
     * Retrieves the total system memory in gigabytes.
     *
     * Queries Prometheus metric `node_memory_MemTotal_bytes` and converts bytes to GB.
     *
     * @returns {Promise<number | null>} Total memory in GB, or null if unavailable.
     */
    static async getTotalMemory(): Promise<number | null> {
        checkEndpoint();

        try {
            const fullUrl = `${urlEndpoint}${urlPath}`;
            const query = 'node_memory_MemTotal_bytes';

            const response = await axios.get(fullUrl, { params: { query } });
            const result = response.data?.data?.result;

            if (!result || result.length === 0) {
                console.warn('No data for node_memory_MemTotal_bytes');
                return null;
            }

            const totalBytes = parseFloat(result[0].value[1]);
            if (isNaN(totalBytes)) return null;

            return totalBytes / (1024 ** 3);
        } catch (err) {
            console.error('Error fetching total memory from Prometheus:', (err as Error).message);
            return null;
        }
    }
    /**
     * Retrieves the used system memory in gigabytes.
     *
     * Calculates used memory by subtracting `node_memory_MemAvailable_bytes`
     * from `node_memory_MemTotal_bytes` and converts bytes to GB.
     *
     * @returns {Promise<number | null>} Used memory in GB, or null if unavailable.
     */
    static async getUsedMemory(): Promise<number | null> {
        checkEndpoint();

        try {
            const fullUrl = `${urlEndpoint}/api/v1/query`;
            const query = 'node_memory_MemTotal_bytes - node_memory_MemAvailable_bytes';

            const response = await axios.get(fullUrl, { params: { query } });
            const result = response.data?.data?.result;

            if (!result || result.length === 0) {
                console.warn('No data for used memory query');
                return null;
            }

            const usedBytes = parseFloat(result[0].value[1]);
            if (isNaN(usedBytes)) return null;

            return usedBytes / (1024 ** 3);
        } catch (err) {
            console.error('Error fetching used memory from Prometheus:', (err as Error).message);
            return null;
        }
    }



}
