import axios from 'axios';
import { checkEndpoint, urlEndpoint, urlPath } from './config';

export class System {
    /**
     * Fetches the OS name (sysname) from node_exporter metrics.
     *
     * @returns {Promise<string | null>} OS name (e.g., "Linux", "Windows_NT") or null if unavailable.
     */
    static async getOsName(): Promise<string | null> {
        checkEndpoint();

        try {
            const query = 'node_uname_info';
            const response = await axios.get(`${urlEndpoint}${urlPath}`, { params: { query } });
            const result = response.data?.data?.result;

            if (!result || result.length === 0) return null;

            return result[0].metric.sysname ?? null;
        } catch (error) {
            console.error('Failed to fetch OS name:', (error as Error).message);
            return null;
        }
    }

    /**
     * Fetches the OS kernel version (release) from node_exporter metrics.
     *
     * @returns {Promise<string | null>} Kernel version string or null if unavailable.
     */
    static async getKernelVersion(): Promise<string | null> {
        checkEndpoint();

        try {
            const query = 'node_uname_info';
            const response = await axios.get(`${urlEndpoint}${urlPath}`, { params: { query } });
            const result = response.data?.data?.result;

            if (!result || result.length === 0) return null;

            return result[0].metric.release ?? null;
        } catch (error) {
            console.error('Failed to fetch kernel version:', (error as Error).message);
            return null;
        }
    }

    /**
     * Fetches the OS kernel build version info (version) from node_exporter metrics.
     *
     * @returns {Promise<string | null>} Kernel build version string or null if unavailable.
     */
    static async getKernelBuildVersion(): Promise<string | null> {
        checkEndpoint();

        try {
            const query = 'node_uname_info';
            const response = await axios.get(`${urlEndpoint}${urlPath}`, { params: { query } });
            const result = response.data?.data?.result;

            if (!result || result.length === 0) return null;

            return result[0].metric.version ?? null;
        } catch (error) {
            console.error('Failed to fetch kernel build version:', (error as Error).message);
            return null;
        }
    }

    /**
     * Fetches the machine architecture from node_exporter metrics.
     *
     * @returns {Promise<string | null>} Machine architecture (e.g., "x86_64") or null if unavailable.
     */
    static async getMachineArchitecture(): Promise<string | null> {
        checkEndpoint();

        try {
            const query = 'node_uname_info';
            const response = await axios.get(`${urlEndpoint}${urlPath}`, { params: { query } });
            const result = response.data?.data?.result;

            if (!result || result.length === 0) return null;

            return result[0].metric.machine ?? null;
        } catch (error) {
            console.error('Failed to fetch machine architecture:', (error as Error).message);
            return null;
        }
    }
    /**
     * Retrieves the system uptime in seconds.
     *
     * Calculated as the difference between
     * `node_time_seconds` and `node_boot_time_seconds` metrics.
     *
     * @returns {Promise<number | null>} Uptime in seconds, or null if unavailable.
     */
    static async getUptimeSeconds(): Promise<number | null> {
        checkEndpoint();
        try {
            const query = 'node_time_seconds - node_boot_time_seconds';
            const response = await axios.get(`${urlEndpoint}${urlPath}`, {
                params: { query }
            });

            const result = response.data.data.result;
            if (!result || result.length === 0) {
                console.warn('No uptime data found in Prometheus');
                return null;
            }

            const uptimeStr = result[0].value[1];
            const uptime = parseFloat(uptimeStr);
            if (isNaN(uptime)) return null;

            return uptime;
        } catch (error) {
            console.error('Failed to fetch system uptime from Prometheus:', (error as Error).message);
            return null;
        }
    }
}
